import { performPCA } from './pca.js';

// Load the CSV data
function loadCSVData() {
  d3.csv("https://raw.githubusercontent.com/ZeynebMhamedi/VizHub/main/full_gea.csv").then((data) => {
    const processedData = processData(data);
    const pcaData = performPCA(processedData);
    createScatterPlot(pcaData);
  }).catch((error) => {
    console.error("Error loading CSV file:", error);
  });
}

function processData(data) {
  // Convert the data from strings to numbers and store it in an array
  const processedData = data.map((row) => {
    return Object.values(row).map((value) => parseFloat(value));
  });

  return processedData;
}

function createScatterPlot(data) {
  // Constants for the visualization
  const margin = { top: 50, right: 50, bottom: 50, left: 50 };
  const width = 800;
  const height = 600;

  // Create the SVG container for the visualization
  const svg = d3
    .select("body")
    .append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom)
    .append("g")
    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

  // Create x and y scales for the scatter plot
  const xScale = d3.scaleLinear().domain(d3.extent(data, (d) => d[0])).range([0, width]);
  const yScale = d3.scaleLinear().domain(d3.extent(data, (d) => d[1])).range([height, 0]);

  // Create the scatter plot using the PCA data
  svg
    .selectAll("circle")
    .data(data)
    .enter()
    .append("circle")
    .attr("cx", (d) => xScale(d[0]))
    .attr("cy", (d) => yScale(d[1]))
    .attr("r", 3)
    .attr("fill", "#1f77b4");

  // Create x and y axes for the scatter plot
  const xAxis = d3.axisBottom(xScale);
  const yAxis = d3.axisLeft(yScale);

  svg.append("g").attr("transform", `translate(0, ${height})`).call(xAxis);
  svg.append("g").call(yAxis);
}

// Call the function
loadCSVData();
